  .inesprg $01   ; 1x 16KB PRG code
  .ineschr $01   ; 1x  8KB CHR data
  .inesmap $00   ; mapper 0 = NROM, no bank swapping
  .inesmir $01   ; background mirroring

;-----------------------------------------------------------------------------------------------------

  .include "VariablesandConstants.asm"

;-----------------------------------------------------------------------------------------------------

;;;;;;;;;;;;
;;;;;;;;;;;;
    
  .bank 0
  .org $8000 

;;;;;;;;;;;;
;;;;;;;;;;;;

RESET:
  SEI          ; disable IRQs
  CLD          ; disable decimal mode
  LDX #$40
  STX $4017    ; disable APU frame IRQ
  LDX #$FF
  TXS          ; Set up stack
  INX          ; now X = 0
  STX $2000    ; disable NMI
  STX $2001    ; disable rendering
  STX $4010    ; disable DMC IRQs

vblankwait
  BIT $2002
  BPL vblankwait

clrmem:
  LDA #$00
  STA $0000, x
  STA $0100, x
  STA $0300, x
  STA $0400, x
  STA $0500, x
  STA $0600, x
  STA $0700, x
  LDA #$FE
  STA $0200, x
  INX
  BNE clrmem

vblankwait2
  BIT $2002
  BPL vblankwait2

  LDA #$00
  STA $2003                         ; set the low byte (00) of the RAM address

  JSR LoadPalette

  JSR LoadSprites

  LDA #$00
  STA Room_To_Load
  JSR Load_Entire_Room

  LDA #$00
  JSR GameStateUpdate

  LDA #%10010000                    ; enable NMI, sprites from Pattern Table 0, background from Pattern Table 1
  STA $2000


;-----------------------------------------------------------------------------------------------------
;-----------------------START MAIN PROGRAM------------------------------------------------------------
;-----------------------------------------------------------------------------------------------------

Forever:
  INC sleeping                     ;wait for NMI

.loop
  LDA sleeping
  BNE .loop                        ;wait for NMI to clear out the sleeping flag

  LDA #$01
  STA updating_background          ;this is for when you are changing rooms or something, not really needed here
                                   ;it will skip the NMI updates so as not to mess with your room loading routines

  JSR strobe_controllers

  JSR GameStateIndirect

  LDA GameState
  CMP GameStateOld
  BEQ .next

  JSR GameStateUpdate

.next

  LDA #$00
  STA updating_background 

  LDA updating_background+1        ;we put this in because the usage bar TURNS ON THE BACKGROUND
  BNE .next1

  JSR showCPUUsageBar              ;this is the correct place for the usage bar...oops.

.next1

  JMP Forever                      ;jump back to Forever, and go back to sleep

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;--------THE FUCKIN' NMI ROUTINE, RECOGNIZE BITCH!---------------------------------------------------;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

NMI:
  PHA                              ;protect the registers
  TXA
  PHA
  TYA
  PHA

nmi_start:

  LDA updating_background          ;check to be sure that the main program isn't busy
  BNE skip_graphics_updates
  
  LDA #$02
  STA $4014                        ;set the high byte (02) of the RAM address, start the transfer

  JSR GameStateNMIIndirect

  LDA #$00                         ;tell the ppu there is no background scrolling
  STA $2005
  STA $2005
  STA $2006
  STA $2006

  LDA #%00011110                   ;enable sprites, enable background, no clipping on left side
  STA $2001

  LDA #$00
  STA sleeping                     ;wake up the main program
  STA updating_background+1

skip_graphics_updates:

  PLA                              ;restore the registers
  TAY 
  PLA
  TAX
  PLA

  RTI                              ;return from interrupt

;-----------------------------------------------------------------------------------------------------
;-----------------------GAME STATE LOADING ROUTINE----------------------------------------------------
;-----------------------------------------------------------------------------------------------------

GameStateIndirect:                 ;indirect jump to main program
  JMP [Main_Pointer]

GameStateNMIIndirect:              ;indirect jump to NMI routine
  JMP [NMI_Pointer]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;00=Main Top Down View
;01=Paused

GameStates:
  .word GameState0,GameState1

GameStateNMIs:
  .word GameStateNMI0,GameStateNMI1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GameStateUpdate:                   ;load the game state and NMI pointers
  LDA GameState
  STA GameStateOld
  ASL A                            ;multiply by two
  TAX

  LDA GameStates,x                 ;Load the Main Program Pointer
  STA Main_Pointer
  LDA GameStates+1,x
  STA Main_Pointer+1

  LDA GameStateNMIs,x
  STA NMI_Pointer
  LDA GameStateNMIs+1,x
  STA NMI_Pointer+1

  RTS  

;-----------------------------------------------------------------------------------------------------
;-----------------------MAIN TOP DOWN VIEW GS #$00----------------------------------------------------
;-----------------------------------------------------------------------------------------------------

GameState0:

  JSR Sprite_Cycling

  LDA joypad1;_pressed
  AND #%00001000 ;up
  BEQ .balls

  INC Room_To_Load
  LDA Room_To_Load
  CMP #$04
  BNE .titties
  
  LDA #$00
  STA Room_To_Load

.titties

  JSR Load_Entire_Room

.balls

  RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;-----NMI ROUTINE, MAIN TOP DOWN VIEW GAME STATE #$00------------------------------------------------;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GameStateNMI0:


  RTS

;-----------------------------------------------------------------------------------------------------
;-----------------------MAIN PROGRAM GS #$01-PAUSED---------------------------------------------------
;-----------------------------------------------------------------------------------------------------

GameState1:

  RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;--------NMI ROUTINE, PAUSED GAME STATE #$01-PAUSED--------------------------------------------------;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GameStateNMI1:

  RTS

;-----------------------------------------------------------------------------------------------------


;-----------------------------------------------------------------------------------------------------
;-----------------------GRAPHICS STUFF----------------------------------------------------------------
;-----------------------------------------------------------------------------------------------------

palette:
  .db $22,$29,$1A,$0F, $22,$36,$17,$0F, $22,$30,$21,$0F, $22,$27,$17,$0F   ;background palette
  .db $22,$04,$1C,$10, $22,$05,$11,$08, $22,$16,$29,$08, $22,$28,$31,$0B   ;sprite palette

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

LoadPalette:

  LDA #$01
  STA updating_background+1

  JSR turn_off_graphics

  LDA $2002                        ; read PPU status to reset the high/low latch
  LDA #$3F
  STA $2006                        ; write the high byte of $23C0 address
  LDA #$00
  STA $2006                        ; write the low byte of $23C0 address
  LDY #$00 
.Loop:
  LDA palette,y
  STA $2007
  INY
  CPY #$20
  BNE .Loop

  RTS

;-----------------------------------------------------------------------------------------------------

Sprites:
  .db $80,$1C,$00,$80,$80,$1C,$01,$88,$80,$1C,$02,$90,$80,$1C,$03,$98
  .db $80,$1C,$00,$A0,$80,$1C,$01,$A8,$80,$1C,$02,$B0,$80,$1C,$03,$B8
  .db $80,$1C,$00,$C0,$80,$1C,$01,$C8,$80,$1C,$02,$D0,$80,$1C,$03,$D8

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

LoadSprites:

  LDX #$00
.loop
  LDA Sprites,x
  STA sprite_RAM,x
  INX
  CPX #$30
  BNE .loop

;-----------------------------------------------------------------------------------------------------

Sprite_Cycling:

  LDA Sprite_Rotation
  BNE .Backwards

.loop1
  LDA sprite_RAM,x          ;copy forwards
  STA sprite_DMA,x
  INX
  BNE .loop1

  INC Sprite_Rotation
  RTS

.Backwards
  LDY #$FC
.loop2                      ;copy backwards
  LDA sprite_RAM,x
  STA sprite_DMA,y
  INX
  INY
  LDA sprite_RAM,x
  STA sprite_DMA,y
  INX
  INY
  LDA sprite_RAM,x
  STA sprite_DMA,y
  INX
  INY
  LDA sprite_RAM,x
  STA sprite_DMA,y
  TYA
  SEC
  SBC #$08
  TAY
  INY
  INX 
  BNE .loop2

  LDA #$00
  STA Sprite_Rotation

  RTS


;-----------------------------------------------------------------------------------------------------
;------Background Loading-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------------------

backgroundpointer:                     ;room data for our room_index to reference to
  .word Room1,Room2,Room3,Room4

;-----------------------------------------------------------------------------------------------------

Load_Entire_Room:

  LDA Room_To_Load
  ASL A
  TAX
  LDA backgroundpointer,x       ;set up the pointer to the room info
  STA Room_Info_Pointer
  LDA backgroundpointer+1,x
  STA Room_Info_Pointer+1

  LDA #$01
  STA updating_background+1     ;make sure that the NMI updates don't inturrupt

  JSR turn_off_graphics

  LDA $2002                     ; read PPU status to reset the high/low latch
  LDA #$20
  STA $2006                     ; write the high byte of $3F00 address
  LDA #$00
  STA $2006                     ; write the low byte of $3F00 address


  LDY #$00
  STY Room_Loader_Counter       ;zero out counters
  STY Meta_Tile_Counter

.loop
  LDY Room_Loader_Counter
  LDA [Room_Info_Pointer],y     ;use the room data to find the next metatile
  ASL A
  TAX
  LDA MetaTileSeta,x
  STA Meta_Tile_ptr
  LDA MetaTileSeta+1,x
  STA Meta_Tile_ptr+1

  LDX Meta_Tile_Counter         ;load the tiles into the PPU
  LDY #$00
  LDA [Meta_Tile_ptr],y
  STA $2007
  INY 
  LDA [Meta_Tile_ptr],y 
  STA $2007
  INY 
  LDA [Meta_Tile_ptr],y         ;load the tiles into the buffer
  STA Meta_Tile_Buffer,x
  INY
  INX
  LDA [Meta_Tile_ptr],y
  STA Meta_Tile_Buffer,x
  INX
  STX Meta_Tile_Counter
  INY

  LDA Room_Loader_Counter        ;this finds our left/right position in the data stream
  AND #%00001111
  LSR A
  STA Att_Temp
  LDA Room_Loader_Counter        ;this finds our up/down position in the data stream
  AND #%11100000
  LSR A
  LSR A
  CLC
  ADC Att_Temp  ;this should be the overall attribute byte index
  STA Att_Temp

  LDA Room_Loader_Counter
  AND #%00000001
  BEQ .leftside                  ;of the attribute 4 meta tile area

  LDA Room_Loader_Counter
  AND #%00010000                 ;of the attribute 4 meta tile area
  BEQ .top
 
  LDX Att_Temp                   ;so we are on the bottom, right side
  LDA BGAttributes,x
  AND #%00111111
  STA BGAttributes,x             ;clear out previous attributes

  LDA [Meta_Tile_ptr],y
  AND #%11000000                 ;store our new data
  ORA BGAttributes,x
  STA BGAttributes,x
  JMP .done

.top
  LDX Att_Temp                   ;top, right side
  LDA BGAttributes,x
  AND #%11110011
  STA BGAttributes,x

  LDA [Meta_Tile_ptr],y
  AND #%00001100
  ORA BGAttributes,x
  STA BGAttributes,x
  JMP .done

.leftside
  LDA Room_Loader_Counter
  AND #%00010000
  BEQ .top1
 
  LDX Att_Temp
  LDA BGAttributes,x              ;bottom, left side
  AND #%11001111
  STA BGAttributes,x

  LDA [Meta_Tile_ptr],y
  AND #%00110000
  ORA BGAttributes,x
  STA BGAttributes,x
  JMP .done

.top1
  LDX Att_Temp                     ;bottom, right side
  LDA BGAttributes,x
  AND #%11111100
  STA BGAttributes,x

  LDA [Meta_Tile_ptr],y
  AND #%00000011
  ORA BGAttributes,x
  STA BGAttributes,x

.done

  LDX Room_Loader_Counter          ;store the collision data
  INY  
  LDA [Meta_Tile_ptr],y
  STA Collision_Data,x

  INC Room_Loader_Counter

  LDX Meta_Tile_Counter            ;if we are ready to load the buffer, do so
  CPX #$20
  BEQ .next
  JMP .loop

.next
  LDX #$00
  STX Meta_Tile_Counter            ;load the buffer
.loop1
  LDA Meta_Tile_Buffer,x
  STA $2007
  INX
  CPX #$20
  BNE .loop1

  LDA Room_Loader_Counter          ;if we're done with the room, move on
  CMP #$F0
  BEQ .next1
  JMP .loop

.next1
  LDA $2002                        ; read PPU status to reset the high/low latch
  LDA #$23
  STA $2006                        ; write the high byte of $23C0 address
  LDA #$C0
  STA $2006                        ; write the low byte of $23C0 address
  LDX #$00
.attloop
  LDA BGAttributes,x               ;write attributes
  STA $2007
  INX
  CPX #$40
  BNE .attloop

  RTS

;;;;;;;;;;;;
;;;;;;;;;;;; 
  
  .bank 1
  .org $A000

;;;;;;;;;;;;
;;;;;;;;;;;;


;-----------------------------------------------------------------------------------------------------

; read_joypad will capture the current button state and store it in joypad1.  
;       Off-to-on transitions will be stored in joypad1_pressed
strobe_controllers:
    lda joypad1
    sta joypad1_old ;save last frame's joypad button states
    
    lda #$01
    sta $4016
    lda #$00
    sta $4016
    
    ldx #$08
.loop:    
    lda $4016
    lsr a
    rol joypad1  ;A, B, select, start, up, down, left, right
    dex
    bne .loop
    
    lda joypad1_old ;what was pressed last frame.  EOR to flip all the bits to find ...
    eor #$FF    ;what was not pressed last frame
    and joypad1 ;what is pressed this frame
    sta joypad1_pressed ;stores off-to-on transitions
    
    rts

;-----------------------------------------------------------------------------------------------------

showCPUUsageBar:

  ldx #%00011111  ; sprites + background + monochrome
  stx $2001
  ldy #21  ; add about 23 for each additional line
.loop
    dey
    bne .loop
  dex    ; sprites + background + NO monochrome
  stx $2001
  rts

;-----------------------------------------------------------------------------------------------------

turn_off_graphics:

  LDA #%00000110   ; disable sprites, disable background, no clipping on left side
  STA $2001
  RTS

;-----------------------------------------------------------------------------------------------------

  .include "Background_Info.asm"

;;;;;;;;;;;;

  .org $FFFA     ;first of the three vectors starts here
  .dw NMI        ;when an NMI happens (once per frame if enabled) the 
                   ;processor will jump to the label NMI:
  .dw RESET      ;when the processor first turns on or is reset, it will jump
                   ;to the label RESET:
  .dw 0          ;external interrupt IRQ is not used in this tutorial

;;;;;;;;;;;;
;;;;;;;;;;;;


  .bank 2
  .org $0000
  .incbin "SpriteMovement.chr"  ;include the sprite graphics data
  .incbin "mario1.chr"          ;include the background data

